Package: tidyselect
Title: Select from a Set of Strings
Version: 1.2.0
Authors@R: c(
    person("Lionel", "Henry", , "lionel@rstudio.com", role = c("aut", "cre")),
    person("Hadley", "Wickham", , "hadley@rstudio.com", role = "aut"),
    person("RStudio", role = c("cph", "fnd"))
  )
Description: A backend for the selecting functions of the 'tidyverse'.  It
    makes it easy to implement select-like functions in your own packages
    in a way that is consistent with other 'tidyverse' interfaces for
    selection.
License: MIT + file LICENSE
URL: https://tidyselect.r-lib.org, https://github.com/r-lib/tidyselect
BugReports: https://github.com/r-lib/tidyselect/issues
Depends: R (>= 3.4)
Imports: cli (>= 3.3.0), glue (>= 1.3.0), lifecycle (>= 1.0.3), rlang
        (>= 1.0.4), vctrs (>= 0.4.1), withr
Suggests: covr, crayon, dplyr, knitr, magrittr, rmarkdown, stringr,
        testthat (>= 3.1.1), tibble (>= 2.1.3)
VignetteBuilder: knitr
ByteCompile: true
Config/testthat/edition: 3
Config/Needs/website: tidyverse/tidytemplate
Encoding: UTF-8
RoxygenNote: 7.2.1
NeedsCompilation: no
Packaged: 2022-10-10 14:09:03 UTC; hadleywickham
Author: Lionel Henry [aut, cre],
  Hadley Wickham [aut],
  RStudio [cph, fnd]
Maintainer: Lionel Henry <lionel@rstudio.com>
Repository: CRAN
Date/Publication: 2022-10-10 19:00:02 UTC
